/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.EditReportFolderViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dojo.parser");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var session					= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PermissionsWidget 		= com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget;

var FOLDER_CLOSED_IMG_SRC			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_closed.gif");
var FOLDER_CLOSED_SHARED_IMG_SRC	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_closed_shared.gif");

/**
 * Displays the Edit Report Folder/Node view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.EditReportFolderViewer",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/EditReportFolderViewer.html"),
	
	startup: function() {
		this.inherited(arguments);
		
		// instantiate widgets used to display tabs
		var nodeList = [];
		nodeList.push(this.layoutContainer);
		nodeList.push(this.mainContentPane);
		nodeList.push(this.tabContainer);
		nodeList.push(this.detailsContentPane);
		if (this._createPermissionsTab) {
			nodeList.push(this.permissionContentPane);
		}
		dojo.parser.instantiate(nodeList);
		this.nodeShared.checked = this._selectedNode.shared;	//fix for IE6
	},
	
	constructor: function(args) {
		this._controller = args.controller;
		this._selectedNode = args.node;
		this._createPermissionsTab = true;
		if (args.createPermissionsTab != null) {
			this._createPermissionsTab = args.createPermissionsTab;
		}
		this._args = args;
	},
	
	postCreate: function() {
		var self = this;
		
		this._initText();
		
		this._msgFormatter = new MessageFormatter(this.messageArea);
		
		this._postServiceExec = null;
		this._postSave = null;
		this._postCancel = null;
		
		this._selectedNodeId = this._selectedNode.itemId;
		this._resourceType = this._selectedNode.nodeType;
		
		this._readOnly = false;
		this._isOwner = false;
		this._isProjectLead = false;
		
		var args = this._args;
		
		if (typeof args.postSave != "undefined") {
			this._postSave = args.postSave;
		}
		
		if (typeof args.postCancel != "undefined") {
			this._postCancel = args.postCancel;
		}
		
		if (typeof this._selectedNodeId == "undefined") {
			this._selectedNodeId = null;
		}
				
		this._getSelectedNode();
		
		this._fillParentNodeControl();
			
		if (this._selectedNodeId != null) {
			var repositoryItemId = this._selectedNodeId;			
			if (this._createPermissionsTab == true) {
				this.permissionsWidget = new PermissionsWidget({
					repositoryItemId: repositoryItemId,
					messageArea: this.messageArea,
					resourceType: this._resourceType,
					parentWidget: this
				});
				this.permissionContentPane.appendChild(this.permissionsWidget.domNode);
			}
		}
		
		this._parentFolderChanged = false;
		this._nameChanged = false;
		this._nodeSharedChanged = false;

		// save and cancel actions
		dojo.connect(this.saveButton, "onclick",
			function(e) {
				e.preventDefault();
				self._handleSave();
				self._parentFolderChanged = false;
				self._nameChanged = false;
				self._nodeSharedChanged = false;	
			}
		);
		
		dojo.connect(this.closeButton, "onclick",
			function(e) {
				e.preventDefault();
				if (self._postCancelFun) {
					self._postCancelFun();
				}
				else {
					self._controller.defaultAction();
				}
			}
		);
		
		var postSave = function() {
			self._controller.editReportNode(self._selectedNode,
				null, self._postSave, self._postCancel);
		};
		
		var postCancel = function() {
			self._controller.editReportNode(self._selectedNode,
				null, self._postSave, self._postCancel);
		};
		
		dojo.connect(this.parentFolder, "onchange",
			function(e) {
				e.preventDefault();
				self._handleParentChange();
				var parentId = (self._selectedNode.parent != null) ? self._selectedNode.parent.itemId : ""; 
				self._parentFolderChanged = (parentId != self.parentFolder.options[self.parentFolder.selectedIndex].value);				
				self._updateSaveButtonState();
			}
		);
		
		dojo.connect(this.nodeName, "onkeyup",
			function(e) {
				e.preventDefault();
				var name = dojo.trim(self.nodeName.value);
				self._nameChanged = (name != self._selectedNode.name) ;
				self._updateSaveButtonState(); 
				if (name.length == 0) {
					self.saveButton.disabled = true;
				}
			}
		);
		
		dojo.connect(this.nodeName, "onkeypress",
			function(e) {				   	
				if(e.charCode == 47 || e.charCode == 92 || e.charCode == 42 || e.charCode == 63 ||
				   e.charCode == 124 || e.charCode == 62 || e.charCode == 60 || e.charCode == 58 || e.charCode == 34){
						e.preventDefault();
				}
			}
		);
		
		dojo.connect(this.nodeShared,"onclick",
			function(e){
				self._nodeSharedChanged = (self.nodeShared.checked != self._selectedNode.shared);
				self._updateSaveButtonState();
			}
		);
				
		if (this._createPermissionsTab == false) {
			this.tabContainer.removeChild(this.permissionContentPane);
		}
	},
	
	_getSelectedNode: function() {
		var widget = dijit.byId(this._selectedNodeId);
		if(widget!=null) {
			this._selectedNode = widget._folder;
		}
		if (this._selectedNode != null) {
			this._isOwner = this._controller.secMgr.isOwner(this._selectedNode);
			this._isProjectLead = this._controller.secMgr.isProjectLead(this._selectedNode);
		}
	},
	
	_initText: function() {
		Util.removeChildren(this.saveButton);
		this.saveButton.appendChild(document.createTextNode(CommonMessages.SAVE_BUTTON_LABEL));
		
		Util.removeChildren(this.closeButton);
		this.closeButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		this.detailsContentPane.title = ReportMessages.EDIT_REPORT_NODE_VIEWER_DETAILS_TAB_TITLE;
		this.permissionContentPane.title = ReportMessages.EDIT_REPORT_NODE_VIEWER_PERMISSIONS_TAB_TITLE;
		
		Util.removeChildren(this.nodeNameLabel);
		this.nodeNameLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_NAME_LABEL));
		
		Util.removeChildren(this.descriptionLabel);
		this.descriptionLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_DESCRIPTION_LABEL));
		
		Util.removeChildren(this.parentFolderLabel);
		this.parentFolderLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_PARENT_FOLDER_LABEL));
		
		Util.removeChildren(this.nodeSharedLabel);
		this.nodeSharedLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_NODE_SHARED_LABEL));
	},
	
	_setPageTitle: function(title) {
		Util.removeChildren(this.pageTitle);
		var pageTitleIcon = document.createElement("img");
		pageTitleIcon.src = this._selectedNode.shared ? FOLDER_CLOSED_SHARED_IMG_SRC : FOLDER_CLOSED_IMG_SRC;
		this.pageTitle.appendChild(pageTitleIcon);
		var pageTitleText = document.createTextNode(title.replace(" ", "\xa0", "g")); // preserve the spaces
		this.pageTitle.appendChild(pageTitleText);
	},
	
	_fillParentNodeControl:  function() {	
		var selectedNode = this._selectedNode;
		
		this.parentFolder.options[0] = new Option(CommonMessages.NO_PARENT_FOLDER_OPTION, "", false, false);			
				
		var selectedNodeParentNodeId = null;
		if (selectedNode != null && selectedNode.parent) {
			selectedNodeParentNodeId = selectedNode.parent.itemId;
		}
		else {
			this.parentFolder.options[0].node = selectedNode;
		}
		
		var parentNodes = this._controller.reportTreeParser.getValidParents(this._selectedNodeId);
		if (parentNodes != null && parentNodes.length > 0) {
			for (var i = 0; i < parentNodes.length; i++) {
				var parentNode = parentNodes[i];
				
				if (parentNode.report) {
					continue;
				}
				
				var parentNodeId = parentNode.itemId;
				
				if (parentNodeId == "" || parentNodeId == null) {
					continue;
				}
				
				if (selectedNode != null && parentNodeId == selectedNode.itemId) {
					continue;
				}
				
				if (parentNodeId != selectedNodeParentNodeId) {
					if (parentNode.shared && !this._controller.secMgr.canShare(parentNode)) {
						continue;
					}
				}
				
				var selected = (selectedNodeParentNodeId != null) && (selectedNodeParentNodeId == parentNodeId);		
				var opt = new Option(parentNode.name, parentNodeId, selected, selected);
				opt.node = parentNode;
				this.parentFolder.options[this.parentFolder.options.length] = opt;
				
				if (selected) {
					opt.selected = true; // fix for IE6
				}
			}
		}
		
		// fill other controls
		if (selectedNode != null) {
			this.nodeName.value = selectedNode.name;
			this.descriptionInput.style.display = "none";
			
			this._setPageTitle(this.nodeName.value);
						
			this.nodeShared.checked = selectedNode.shared;
						
			var self = this;
			var callback = function(editable) {
				if (editable == false) {
					self._readOnly = true;
					self._disableControls();
				}
			};
			this._controller.secMgr.canEdit(selectedNode, callback);
		}
		
		this._checkControlStates();
	},
	
	_disableControls: function() {
		this.saveButton.disabled = true;
		this.nodeName.disabled = true;
		this.description.disabled = true;
		this.parentFolder.disabled = true;
		this.nodeShared.disabled = true;
	},
	
	_handleParentChange: function() {
		this._checkControlStates();
	},
	
	_checkControlStates: function() {
		var shared = false;
		var disabled = false;
		
		if (!this._controller.secMgr.canShare(this._selectedNode)) {
			disabled = true;
		}
		
		if (this.parentFolder.selectedIndex >= 0) {
			var opt = this.parentFolder.options[this.parentFolder.selectedIndex];
			if (opt.node) {
				shared = opt.node.shared;
			}
			if (opt.value != "") {
				disabled = true;
			}
		}
		
		this.nodeShared.disabled = disabled;
		this.nodeShared.checked = shared;
	},
	
	_updateSaveButtonState: function() {
		var disabled = true;
		if (this._nameChanged) {
			disabled = false;
		}
		else if (this._parentFolderChanged) {
			disabled = false;
		}
		else if (this._nodeSharedChanged) {
			disabled = false;
		} 
		else if (this.permissionsWidget && this.permissionsWidget._permissionsChanged) {
			disabled = false;
		}
		this.saveButton.disabled = disabled;
	},
	
	_handleSave: function() {
		Util.removeChildren(this.messageArea);
		
		var name = this.nodeName.value;
  		for (var i = 0; i < name.length; i++) {
			if(name.charCodeAt(i) == 47 || name.charCodeAt(i) == 92 || name.charCodeAt(i) == 42 || name.charCodeAt(i) == 63 ||
			   name.charCodeAt(i) == 124 || name.charCodeAt(i) == 62 || name.charCodeAt(i) == 60 || name.charCodeAt(i) == 58  || name.charCodeAt(i) == 34){
				this._displayErrorMessage(CommonMessages.ERROR_FOLDER_NAME+ ": * \\\ / | \  : < \" > ? ");
				return;
			}
		}
		
		var shared = this.nodeShared.checked;
		var parentFolderId = "";
		
		if (this.parentFolder.selectedIndex >= 0) {
			parentFolderId = this.parentFolder.options[this.parentFolder.selectedIndex].value;
		}

		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		
		var owner = session.getAuthenticatedContributor();
		
		var self = this;
		
		var postExec = function() {
			self._controller.syncReportNodeChanged();
			if (self._postSave) {
				self._postSave();
			}
			self._getSelectedNode();
			self._selectedNode.shared = self.nodeShared.checked;
			self._setPageTitle(self.nodeName.value);
			self._controller.reportTreeParser.expandFolder(parentFolderId);
		};
		
		this._postServiceExec = postExec;
		
		if (this._selectedNodeId) {
			var selectedNode = this._selectedNode;
			
			// if the shared flag changed, we need to update all the child nodes
			var sharedChanged = (shared != selectedNode.shared);
			 			
			this._errorMsg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_REPORT_FOLDER, [dojo.trim(name)]);
						
			var srh = new ServiceResponseHandler(this, "_success", "_remoteError");
			var params = {id: selectedNode.itemId, name: name, shared: shared, parentFolderId: parentFolderId, 
		 				  projectAreaId: projectArea.itemId};
			ReportClient.postUpdateReportTreeFolder(srh, params);	
		}
		this.saveButton.disabled = true;
	},
	
	_success: function() {
		Util.removeChildren(this.messageArea);
		if (this._postServiceExec != null) {
			this._postServiceExec();
		}
		if (this.permissionsWidget != null) {
			this.permissionsWidget._updateAclEntries();
		}
		this.saveButton.disabled = true;
	},
	
	_remoteError: function(error) {
		this._msgFormatter.displayErrorMessage(this._errorMsg, error);
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);	
	}
	
});

})();